-- ##### TerraLAB - www.terralab.ufop.br
-- Computer Science Departament, Federal Univeristy of Ouro Preto
-- TerraME Prey-Predator dynamic model based on differental equations
-- Tiago Garcia de Senna Carneiro - 07/2012
-------------
-- Each cell has different prey and predator populations and the same environmental constrains.
-- However, predators migrate to cells with more preys.
-------------


--#PARAMETERS
birthPreyRate = 0.2
predationRate = 0.01 -- death prey rate
birthPredatorPerPreyRate = 0.01 -- birth predator rate
deathPredatorRate =  0.1 -- death prey rate
predatorMigrationRate = 0.7 -- predator migration rate
timeStep =  1 -- See the effects of 0.25, 0.125, 0.0625, and 0.03125
INTEGRATION_METHOD = integrationHeun --  vary the integration Method: ("integrationHeun"), "integrationRungeKutta", "integrationEuler"

--#SPATIAL MODEL
rand = Random{ seed = 1 }
cs = CellularSpace{ xdim = 10 }
forEachCell( cs, function(cell,i) 
	cell.id = i
	cell.preys = 100 * rand:number(0,1)
	cell.predators = 10 * rand:number(0,1)
end )
cs:createNeighborhood()
cs:synchronize()

--#LEGENDS
preysLeg = Legend{
		-- Attribute name:  height
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 150,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "green", value = 150}
		}
}


predatorsLeg = Legend{
		-- Attribute name:  soilWater
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 100,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "red", value = 100}
		}
}

--#OBSERVERS
Observer{ subject = cs, type = "map", attributes={"preys", "predators"},legends= { preysLeg, predatorsLeg} }


--#RULES
for t = 0, 4000, timeStep do 

	forEachCell( cs, function( cell )
		--print(cell.id, cell.preys, cell.predators)
		cell.preys, cell.predators = d{
			{ 	
				function( t, q )  
					return q[1]*birthPreyRate - q[1]*q[2]*predationRate 
				end , 
				function( t, q ) 
					return q[2]*q[1]*birthPredatorPerPreyRate - q[2]*deathPredatorRate
				end
			},
			{ cell.past.preys, cell.past.predators }, 
			0, timeStep, 0.03125 
		}
		--print(cell.id, cell.preys, cell.predators)
	end)
	cs:synchronize()	-- the process are sequential in time (migration and growth proccesses)

					
	-- migration of predator to cell with more preys
	forEachCell( cs, function(cell)
		local maxPreys = cell.preys
		local maxNeigh = cell
		forEachNeighbor( cell, function (cell, neigh ) 
			local preyDensity = neigh.preys
			if( preyDensity > maxPreys ) then 
				maxPreys = preyDensity
				maxNeigh = neigh
			end
		end)
		if( cell ~= maxNeigh) then 
			--print("Before: ",cell.predators, maxNeigh.predators)
			cell.predators, maxNeigh.predators = d{
				{ 	-- flow from the cell 
					function( t, q )  
						return  - predatorMigrationRate*q[1] 
					end , 
					-- flow to the neighbor
					function( t, q ) 
						return predatorMigrationRate*q[1]
					end
				},
				{ cell.past.predators, maxNeigh.past.predators }, 
				0, timeStep, 0.03125 
			}	
			--print("After: ",cell.predators, maxNeigh.predators, "- from: ", cell.id, " to: ", neigh.id)
		end				
	end)
	
	cs:synchronize() -- the process are sequential in time (migration and growth proccesses)
	
	cs:notify()
	
	print("Time: ", t)
end
